# Serene Bach plugin for fold category.
#
# Release 1.00 (December  9,2005)
#         1.01 (February 12,2006) corrected trouble to which the number of categories was not counted.
#
# This software is provided as-is. You may use it for commercial or 
# personal use. If you distribute it, please keep this notice intact.
#
# Copyright (c) 2005-2006 Yujiro Araki

package sb::Plugin::FoldCategory;

use sb::Plugin ();
use vars qw($js,@history);

sb::Plugin->register_plugin(
    'lang' => {
        'ja' => 'euc',
        'en' => 'ascii',
    },
    'text' => {
        'type'    => 'cms',
        'name'    => 'FoldCategory',
        'text'    => 'Fold Category List',
        'author'  => 'Yujiro Araki',
        'detail'  => 'http://www.koikikukan.com/',
        'version' => '1.00',
    },
    'data' => 0,
    'file' => 'foldcategory.txt',
);

# register callback function to cms entry point
sb::Plugin->register_content_module(
    'type'     => 'main',
    'callback' => \&sb::Content::FoldCategory::_fold_category_list,
    'field'    => 'fold_category',
);

package sb::Content::FoldCategory;

sub _fold_category_list { # JeS[Xg
    my $cms = shift;
    my %var = @_;

    if ($var{'mode'} eq 'ent') {
        $var{'entryinfo'} = &_entry_info(%var);
    }

    my ($all_list,$sub_list);
    my $cat = undef;
    $js = '';
    $all_list = &_category_tree(
        'cat' => $var{'sortedcat'},
        'num' => $var{'entryinfo'}{'category'},
    );

    $cat = $var{'cat'}->{$var{'id'}} if ($var{'mode'} eq 'cat');
    if ($var{'mode'} eq 'ent' and $var{'entry'}->[0]) {
        my $cat_id = $var{'entry'}->[0]->cat;
        $cat = $var{'cat'}->{$cat_id} if ($cat_id ne '');
    }
    if ($cat) {
        $sub_list = &_category_tree(
            'cat'    => $var{'sortedcat'},
            'branch' => $cat->id,
            'num'    => $var{'entryinfo'}{'category'},
        );
    }
    $js = "\n".'<script type="text/javascript">'."\n<!--\n".$js."//-->\n</script>\n";
    $cms->num(0);
    $cms->tag('fold_category_list' => ($all_list) ? '<ul>' . $all_list . '</ul>' . $js : '');
    $cms->tag('fold_category_script' => '<script type="text/javascript" src="'.$var{'conf'}->value('conf_srv_base').'menufolder.js" charset="utf-8"></script>');
    return 1;
}

sub _category_tree { # JeS[c[
    my %param = (
        'cat'    => [],
        'branch' => undef,
        'num'    => {},
        'no_num' => undef,
        'counter' => undef,
        @_
    );
    my $list = '';
    my $same;
    if(!defined($param{'counter'})){
        $param{'counter'} = 0;
        push(@history,$param{'counter'});
    }
    foreach $i (@history){
        if($param{'counter'} == $i) {
            $same = 1;
        }
    }
    if($same){
        $param{'counter'}*=100;
    }

    foreach my $cat ( @{$param{'cat'}} ) {
        next if ($cat->get_option('list') == 1);
        next if (!defined($param{'branch'}) and $cat->main ne '');
        next if ( defined($param{'branch'}) and $cat->main ne $param{'branch'});
        my $text = $cat->text; # code by Fuco. Thanks!
        $text = ($text ne '') ? ' title="' . sb::Text->entitize($text) . '"' : '';
        if ($cat->sub ne '') {
            $list .= '<li><div class="subcategories" id="subcategories' . $param{'counter'} . 'name"><a href="' . $cat->cat_url . '"' . $text . '>' . $cat->name . '</a>';
        } else {
            $list .= '<li><a href="' . $cat->cat_url . '"' . $text . '>' . $cat->name . '</a>';
        }
        $list .= ' (' . int($param{'num'}->{$cat->id}) . ')' if (!$param{'no_num'});
        if ($cat->sub ne '') {
            $list .= '</div>';
        }
        if ($cat->sub ne '') {
            $list .= "\n" . '<div id="subcategories' . $param{'counter'} . 'list"><ul>';
            $js .= "FoldNavigation('subcategories" . $param{'counter'} . "','on',false);\n";
            $param{'counter'}++;
            push(@history,$param{'counter'});
            $list .= &_category_tree(
                'cat'     => $param{'cat'},
                'branch'  => $cat->id,
                'num'     => $param{'num'},
                'no_num'  => $param{'no_num'},
                'counter' => $param{'counter'},
            );
            $list .= '</ul></div>';
        }
        $list .= '</li>' . "\n";
    }
    return($list);
}

sub _entry_info { # eGg[̎擾
	my %var = @_;
	my %info = (
		'latest'   => [],
		'monthly'  => {},
		'daily'    => {},
		'neighbor' => {'prev'=>undef,'next'=>undef,},
		'category' => {},
	);
	my $check = -1;
	my @array = sb::Data->load('Entry','cond'=>{'stat'=>[1,2]},'sort'=>'date','order'=>1);
	for (my $i=0;$i<@array;$i++) {
		my $entry = $array[$i];
		$info{'category'}{$entry->cat}++ if ($entry->cat ne ''); # JeS[̃JEg
		if ($entry->add ne '') { # ֘AJeS[̃JEg
			foreach ( split(',',$entry->add) ) {
				next if ($_ eq '');
				$info{'category'}{$_}++;
			}
		}
		if ( ($var{'mode'} eq 'ent' and $var{'id'} == $entry->id)
		  or ($var{'mode'} eq 'mob' and $var{'id'} ne '' and $var{'id'} == $entry->id) ) {
			$check = $i; # ÕGg[i[
		}
	}
	$info{'neighbor'}{'next'} = $array[$check - 1] if ($check > 0);
	$info{'neighbor'}{'prev'} = $array[$check + 1] if ($check > -1 and $check < $#array);
	return(\%info);
}

1;
__END__
